<?php

namespace Drupal\instagram_media\Plugin\Block;

use Drupal\block\BlockForm;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Template\Attribute;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use GuzzleHttp\Exception\RequestException;

/**
 * Provides a Instragram feed block.
 *
 * @Block(
 *   id = "instagram_media",
 *   admin_label = @Translation("Instagram Media Block"),
 * )
 */
class InstagramMedia extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The http client provider.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The tempstore service.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Configuration.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The file system service.Use different styles for posts
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new Instagram Media Block.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The http client provider.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   Private temporary storage factory.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   The configuration.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger channel.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ClientInterface $http_client,
    AccountProxyInterface $currentUser,
    PrivateTempStoreFactory $temp_store_factory,
    RequestStack $request_stack,
    LanguageManagerInterface $language_manager,
    ConfigFactoryInterface $config,
    FileSystemInterface $file_system,
    LoggerChannelInterface $logger,
    ModuleHandlerInterface $module_handler,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->httpClient = $http_client;
    $this->currentUser = $currentUser;
    $this->tempStoreFactory = $temp_store_factory->get('instagram_media');
    $this->requestStack = $request_stack;
    $this->languageManager = $language_manager;
    $this->configFactory = $config;
    $this->fileSystem = $file_system;
    $this->logger = $logger;
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('http_client'),
      $container->get('current_user'),
      $container->get('tempstore.private'),
      $container->get('request_stack'),
      $container->get('language_manager'),
      $container->get('config.factory'),
      $container->get('file_system'),
      $container->get('logger.factory')->get('instagram_media'),
      $container->get('module_handler'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'token' => '',
      'image_style_options' => 'image_style',
      'image_style' => 'large',
      'responsive_image_style' => 'wide',
      'limit' => 6,
      'different_image_style' => 0,
      'different_responsive_style' => 0,
      'autoplay' => TRUE,
      'video' => FALSE,
      'post_caption' => FALSE,
      'media' => '',
      'header_mode' => 'tag_and_link',
      'insights' => FALSE,
      'links' => FALSE,
      'swiper' => FALSE,
      'swiper_arrow' => FALSE,
      'swiper_pagination' => FALSE,
      'fancybox' => FALSE,
      'disabled_style' => FALSE,
      'grid' => 'no-grid',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $style_access = $this->moduleHandler->moduleExists('image');
    $responsive_style_access = $this->moduleHandler->moduleExists('responsive_image');

    $build = [
      '#theme' => 'instagram_media_block',
      '#autoplay' => $config['autoplay'] ?: FALSE,
      '#video' => $config['video'] ?: FALSE,
      '#post_caption' => $config['post_caption'] ?: FALSE,
      '#insights' => $config['insights'] ?: FALSE,
      '#grid' => $config['grid'] ?: 'no-grid',
      '#swiper_config' => [
        'swiper' => $config['swiper'] ?: FALSE,
        'swiper_arrow' => $config['swiper_arrow'] ?: FALSE,
        'swiper_pagination' => $config['swiper_pagination'] ?: FALSE,
      ],
      '#header_mode' => $config['header_mode'] ?: 'tag_and_link',
      '#fancybox' => $config['fancybox'] ?: FALSE,
    ];

    try {
      $token = $config['token'];
      $limit = $config['limit'] ?: 6;
      $facebookPageId = $this->getInstagramBusinessId($token);

      $instaResponse = $this->httpClient->request('GET',
        "https://graph.facebook.com/v21.0/$facebookPageId", [
          'query' => [
            'fields' => 'instagram_business_account',
            'access_token' => $token,
          ],
        ])->getBody();

      $instagramBusinessId = json_decode($instaResponse,
        TRUE)['instagram_business_account']['id'] ?? NULL;

      if (!$instagramBusinessId) {
        throw new \Exception('Instagram Business ID not found.');
      }

      $mediaRequest = $this->httpClient->request('GET',
        "https://graph.facebook.com/v21.0/$instagramBusinessId/media", [
          'query' => [
            'fields' => 'id,media_type,media_url,permalink,caption,like_count,comments_count,timestamp,thumbnail_url',
            'access_token' => $token,
            'limit' => max($limit, 25),
          ],
        ]);

      $posts = json_decode($mediaRequest->getBody()->getContents(),
        TRUE)['data'];
      $data = [];
      $count = 0;

      if (is_dir($config['media_path'])) {
        $this->fileSystem->deleteRecursive($config['media_path']);

        $this->logger->info('Instagram Media media data for %block_id has been purged.',
          [
            '%block_id' => $config['block_id'],
          ]);
      }
      $this->fileSystem->mkdir($config['media_path'], NULL, TRUE);

      foreach ($posts as $key => &$post) {
        if ($count >= $limit) {
          break;
        }
        $post['attributes'] = new Attribute([
          'class' => [],
          'alt' => $post['caption'] ?? '',
        ]);
        $media_url = $config['media_path'] . $this->getFileName($post['media_url']);
        $dataMedia = (string) \Drupal::httpClient()->get($post['media_url'])->getBody();
        $file = \Drupal::service('file.repository')->writeData(
          $dataMedia,
          $media_url,
          FileSystemInterface::EXISTS_REPLACE
        );

        $media_uri = $file->getFileUri();

        $type = $post['media_type'];


        if (!empty($config['fancybox'])) {
          $post['fancybox_full_uri'] = \Drupal::service('file_url_generator')
            ->generateString($media_uri);
        }
        if ($type === 'VIDEO' && $config['video']) {
          continue;
        }
        elseif ($type === 'VIDEO') {
          $post['video_class'][] = 'insta-video';
          $post['media_url'] = \Drupal::service('file_url_generator')
            ->generateString($media_uri);
        }
        else {
          // Processing image
          $entity = [
            '#theme' => 'image',
            '#uri' => $media_uri,
            '#attributes' => $post['attributes'],
          ];
          if ($config['image_style_options'] === 'image_style') {
            if (!empty($config['image_style']) && $style_access) {
              $entity = [
                '#theme' => 'image_style',
                '#style_name' => $config['image_style'],
                '#uri' => $media_uri,
                '#attributes' => $post['attributes'],
              ];
            }

            if ($this->configuration['different_image_style'] && !empty($this->configuration['image_styles_rendering']["image_style_$key"])) {
              $entity = [
                '#theme' => 'image_style',
                '#style_name' => $this->configuration['image_styles_rendering']["image_style_$key"],
                '#uri' => $media_uri,
                '#attributes' => $post['attributes'],
              ];
            }
          }
          elseif ($config['image_style_options'] === 'responsive_image_style') {
            if (!empty($config['responsive_image_style']) && $responsive_style_access) {
              $entity = [
                '#theme' => 'responsive_image',
                '#uri' => $media_uri,
                '#responsive_image_style_id' => $config['responsive_image_style'],
                '#attributes' => $post['attributes'],
              ];
            }
            if ($this->configuration['different_responsive_style'] && !empty($this->configuration['responsive_image_rendering']["responsive_style_$key"])) {
              $entity = [
                '#theme' => 'responsive_image',
                '#uri' => $media_uri,
                '#responsive_image_style_id' => $this->configuration['responsive_image_rendering']["responsive_style_$key"],
                '#attributes' => $post['attributes'],
              ];
            }
          }
          $post['media_url'] = $entity;

          // Attributes
          $post['attributes']['class'][] = 'insta-image';
          $post['attributes']['loading'] = 'lazy';
        }

        if (isset($post['thumbnail_url'])) {
          $thumbnail_url = $config['media_path'] . $this->getFileName($post['thumbnail_url']);
          $dataThumbnail = (string) \Drupal::httpClient()->get($post['thumbnail_url'])->getBody();
          $file = \Drupal::service('file.repository')->writeData(
            $dataThumbnail,
            $thumbnail_url,
            FileSystemInterface::EXISTS_REPLACE
          );

          $post['thumbnail_url'] = \Drupal::service('file_url_generator')
            ->generateString($file->getFileUri());

        }
        $count++;
        $data[] = $post;
      }

      if (isset($config['block_id'])) {
        $block = $this->entityTypeManager->getStorage('block')
          ->load($config['block_id']);
        $block->getPlugin()->setConfigurationValue('media', serialize($data));
        $block->save();
      }
      if ($config['links']) {
        $build['#user_links'] = $this->getInstagramLinks($instagramBusinessId,
          $token);
      }
      if (empty($config['disabled_style'])) {
        $build['#attached']['library'][] = 'instagram_media/instagram';
      }

      $build['#posts'] = $data;
      $this->attachLibraries($build, $config);
      \Drupal::moduleHandler()->alter('build', $build);

      return $build;
    }
    catch (\Exception $e) {
      $this->logger->error('Error retrieving Instagram media: @message',
        ['@message' => $e->getMessage()]);
      return [];
    }
  }

  protected function getInstagramBusinessId($token) {
    $cid = 'instagram_business_id_' . md5($token);

    $cached = \Drupal::cache()->get($cid);

    if ($cached && !empty($cached->data)) {
      return $cached->data;
    }

    try {
      $response = $this->httpClient->request('GET',
        'https://graph.facebook.com/v21.0/me/accounts', [
          'query' => [
            'access_token' => $token,
            'fields' => 'id',
          ],
        ]);

      $data = json_decode($response->getBody()->getContents(), TRUE);
      $instagramBusinessId = $data['data'][0]['id'] ?? NULL;

      if ($instagramBusinessId) {
        \Drupal::cache()->set($cid, $instagramBusinessId, time() + 86400);
        return $instagramBusinessId;
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('instagram_media')
        ->error('Failed to retrieve Instagram Business ID: @message',
          ['@message' => $e->getMessage()]);
    }

    return NULL;
  }

  /**
   * Get file name from URL.
   */
  protected function getFileName($url) {
    $info = pathinfo(parse_url($url)['path']);
    return $info['basename'];
  }

  public function getInstagramLinks($instagramBusinessId, $token) {
    try {
      $response = \Drupal::httpClient()->request('GET',
        'https://graph.facebook.com/v21.0/' . $instagramBusinessId, [
          'query' => [
            'access_token' => $token,
            'fields' => 'name,username,profile_picture_url,media_count,followers_count,follows_count',
          ],
        ]);
      $data = json_decode($response->getBody()->getContents(), TRUE);

      $profile_picture_url = $data['profile_picture_url'] ?? \Drupal::service('file_url_generator')
        ->generateAbsoluteString(\Drupal::service('extension.list.module')
            ->getPath('instagram_media') . '/misc/images/placeholder-avatar.jpg');

      return [
        'name' => $data['name'] ?? '',
        'username' => $data['username'] ?? '',
        'profile_url' => $data['username'] ? 'https://www.instagram.com/' . $data['username'] . '/' : '',
        'profile_picture_url' => $profile_picture_url,
        'media_count' => $data['media_count'] ?? 0,
        'followers_count' => $this->formatCount($data['followers_count'] ?? 0),
        'follows_count' => $this->formatCount($data['follows_count'] ?? 0),
      ];
    }
    catch (RequestException $e) {
      \Drupal::logger('instagram_media')
        ->error('Failed to retrieve Instagram username: @message',
          ['@message' => $e->getMessage()]);
      return NULL;
    }
  }

  protected function formatCount($count) {
    if ($count >= 1000000) {
      return round($count / 1000000, 1) . t('M', [],
          ['context' => 'instagram_media']);
    }
    if ($count >= 1000) {
      return round($count / 1000, 1) . t('K', [],
          ['context' => 'instagram_media']);
    }
    return $count;
  }

  private function attachLibraries(&$build, $config) {
    if (!empty($config['disabled_style'])) {
      $build['#attached']['library'][] = 'instagram_media/instagram';
    }
    if (!empty($config['fancybox'])) {
      $active_theme = \Drupal::service('theme.manager')
        ->getActiveTheme()
        ->getName();
      $build['#attached']['library'][] = "$active_theme/fancybox";
    }
    if (!empty($config['swiper'])) {
      $active_theme = \Drupal::service('theme.manager')
        ->getActiveTheme()
        ->getName();
      $build['#attached']['library'][] = "$active_theme/swiper";
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return Cache::mergeTags(
      parent::getCacheTags(),
      $this->configFactory->get('instagram_media.settings')->getCacheTags()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['#prefix'] = '<div id="instagram-form--settings">';
    $form['#suffix'] = '</div>';
    $config = $this->getConfiguration();

    $responsive_style_access = $this->moduleHandler->moduleExists('responsive_image');

    $form['token'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Instagram Token'),
      '#rows' => 3,
      '#cols' => 5,
      '#required' => TRUE,
      '#default_value' => $config['token'] ?: '',
      '#description' => $this->t('Paste long lived access token.'),
      '#description_display' => 'after',
    ];

    //    Image and Video settings
    $form['media_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Media settings'),
      '#attributes' => [
        'open' => TRUE,
      ],
    ];

    $form['media_settings']['image_style_options'] = [
      '#type' => 'select',
      '#title' => $this->t('Image style options'),
      '#options' => [
        '' => t('- None -'),
        'image_style' => $this->t('Image style'),
        'responsive_image_style' => $this->t('Responsive image style'),
      ],
      '#default_value' => $config['image_style_options'] ?: '',
      '#attributes' => ['class' => ['image_style_options']],
      '#ajax' => [
        'callback' => [$this, 'formSettingsAjax'],
        'event' => 'change',
        'wrapper' => 'instagram-form--settings',
        'progress' => ['type' => 'throbber'],
      ],
    ];

    $form['media_settings']['image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Image style'),
      '#options' => image_style_options(),
      '#default_value' => $config['image_style'],
      '#states' => [
        'visible' => [
          '.image_style_options' => ['value' => 'image_style'],
        ],
      ],
    ];

    $form['media_settings']['different_image_style'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use different styles for posts'),
      '#default_value' => $this->configuration['different_image_style'],
      '#attributes' => ['class' => ['different_image_style']],
      '#ajax' => [
        'callback' => [$this, 'formSettingsAjax'],
        'event' => 'change',
        'wrapper' => 'instagram-form--settings',
        'progress' => [
          'type' => 'throbber',
        ],
      ],
      '#states' => [
        'visible' => [
          '.image_style_options' => ['value' => 'image_style'],
        ],
      ],
    ];

    // IMAGE STYLE
    $form['image_styles_rendering'] = [
      '#type' => 'details',
      '#title' => $this->t('Image styles'),
      '#description_display' => 'none',
      '#attributes' => [
        'open' => TRUE,
      ],
      '#states' => [
        'visible' => [
          '.image_style_options' => ['value' => 'image_style'],
          'and',
          '.different_image_style' => ['checked' => TRUE],
        ],
      ],
      'styles' => $this->generateImageStyleSelect($this->configuration['limit'],
        []),
    ];

    // RESPONSIVE
    if ($responsive_style_access) {
      $form['media_settings']['responsive_image_style'] = [
        '#type' => 'select',
        '#title' => $this->t('Responsive image style'),
        '#options' => $this->getResponsiveStylesImage(),
        '#default_value' => $config['responsive_image_style'] ?? '',
        '#attributes' => ['class' => ['responsive_image_style']],
        '#states' => [
          'visible' => [
            '.image_style_options' => ['value' => 'responsive_image_style'],
          ],
        ],
      ];

      // SELECTS
      $form['responsive_image_rendering'] = [
        '#type' => 'details',
        '#title' => $this->t('Responsive styles'),
        '#attributes' => [
          'open' => TRUE,
        ],
        '#states' => [
          'visible' => [
            '.image_style_options' => ['value' => 'responsive_image_style'],
            'and',
            '.different_responsive_style' => ['checked' => TRUE],
          ],
        ],
        'styles' => $this->generateResponsiveStyleSelect($this->configuration['limit']),
      ];

      // CHECKBOX
      $form['media_settings']['different_responsive_style'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Use different responsive styles for posts'),
        '#default_value' => $this->configuration['different_responsive_style'] ?? FALSE,
        '#attributes' => ['class' => ['different_responsive_style']],
        '#ajax' => [
          'callback' => [$this, 'formSettingsAjax'],
          'event' => 'change',
          'wrapper' => 'instagram-form--settings',
          'progress' => [
            'type' => 'throbber',
          ],
        ],
        '#states' => [
          'visible' => [
            '.image_style_options' => ['value' => 'responsive_image_style'],
          ],
        ],
      ];
    }

    $form['media_settings']['limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Number of images to display'),
      '#default_value' => $config['limit'] ?? 6,
      '#attributes' => [
        'min' => '1',
      ],
      '#ajax' => [
        'callback' => [$this, 'formSettingsAjax'],
        'event' => 'change',
        'wrapper' => 'instagram-form--settings',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Build image styles count ...'),
        ],
      ],
    ];

    $form['media_settings']['video'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Hide videos'),
      '#required' => FALSE,
      '#default_value' => $config['video'] ?? FALSE,
      '#attributes' => ['class' => ['instagram_media_video']],
    ];

    $form['media_settings']['autoplay'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Autoplay Instagram Videos'),
      '#required' => FALSE,
      '#default_value' => $config['autoplay'] ?? FALSE,
      '#states' => [
        'visible' => [
          '.instagram_media_video' => ['checked' => FALSE],
        ],
      ],
    ];

    // Post settings

    $form['post_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Post settings'),
    ];

    $form['post_settings']['post_caption'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display Instagram post caption'),
      '#default_value' => $config['post_caption'] ?? FALSE,
    ];

    $form['post_settings']['insights'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('View insights'),
      '#default_value' => $config['insights'] ?? FALSE,
    ];

    $form['post_settings']['links'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('View links'),
      '#default_value' => $config['links'] ?? FALSE,
    ];

    $header_mode_options = [
      'only_link' => $this->t('Only Link'),
      'tag_and_link' => $this->t('Tag and Link'),
      'all_info' => $this->t('Looks like Instagram'),
    ];

    $form['post_settings']['header_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Header Mode'),
      '#options' => $header_mode_options,
      '#default_value' => $config['header_mode'] ?: 'tag_and_link',
      '#states' => [
        'visible' => [
          ':input[name="settings[post_settings][links]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Styles settings

    $form['styling_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Styling settings'),
    ];

    $form['styling_settings']['swiper'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Has swiper'),
      '#default_value' => $config['swiper'] ?? FALSE,
      '#attributes' => ['class' => ['instagram_media_video']],

    ];

    $form['styling_settings']['swiper_arrow'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Swiper arrow'),
      '#default_value' => $config['swiper_arrow'] ?? FALSE,
      '#states' => [
        'visible' => [
          '.instagram_media_video' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['styling_settings']['swiper_pagination'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Swiper pagination'),
      '#default_value' => $config['swiper_pagination'] ?? FALSE,
      '#states' => [
        'visible' => [
          '.instagram_media_video' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['styling_settings']['fancybox'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Fancybox'),
      '#default_value' => $config['fancybox'] ?? FALSE,
    ];

    $form['styling_settings']['disabled_style'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Disabled css styles'),
      '#required' => FALSE,
      '#default_value' => $config['disabled_style'] ?: '',
      '#description' => $this->t('Disables all styling to overwrite custom styles'),
      '#attributes' => ['class' => ['disabled_style']],
    ];

    $grid_options = [
      'no-grid' => $this->t('No grid'),
      'grid-4' => $this->t('Grid 4'),
      'grid-2-4' => $this->t('Grid 2-4'),
      'grid-5' => $this->t('Grid 5'),
      'grid-2-5' => $this->t('Grid 2-5'),
      'grid-6' => $this->t('Grid 6'),
      'grid-2-6' => $this->t('Grid 2-6'),
    ];

    $form['styling_settings']['grid'] = [
      '#type' => 'select',
      '#title' => $this->t('Grid settings'),
      '#options' => $grid_options,
      '#description' => 'The grid option, from the mobile version to the PC. That is, 2 in mobile, and 6 in PC',
      '#required' => FALSE,
      '#default_value' => $config['grid'] ?: 'no-grid',
      '#states' => [
        'invisible' => [
          '.disabled_style' => ['checked' => TRUE],
        ],
      ],
    ];

    if ($form_state instanceof SubformStateInterface) {
      $form_state = $form_state->getCompleteFormState();
      $values = $form_state->getValues();

      if (!empty($values['settings'])) {
        // IMAGE STYLES

        $form['image_styles_rendering']['styles'] = $this->generateImageStyleSelect($values['settings']['media_settings']['limit'],
          $values['settings']);

        // RESPONSIVE IMAGE STYLES

        if ($responsive_style_access) {
          $form['responsive_image_rendering']['styles'] = $this->generateResponsiveStyleSelect($values['settings']['media_settings']['limit'],
            $values['settings']);
        }
      }
    }
    $form_state->setRebuild(TRUE);
    return $form;
  }

  /**
   * Return renderable array of image styles
   *
   * @param integer $count
   *
   * @return array
   */

  public function generateImageStyleSelect(int $count, array $values) {
    $styles = image_style_options();
    $container = [];

    for ($i = 0; $i < $count; $i++) {
      if (empty($values)) {
        $value = @$this->configuration['image_styles_rendering']["image_style_$i"];
      }
      else {
        $value = $values['image_styles_rendering']['styles']["image_style_$i"];
      }

      $container["image_style_$i"] = [
        '#type' => 'select',
        '#title' => $this->t('Image style for post @count', ['@count' => $i]),
        '#default_value' => !empty($value) ? $value : NULL,
        '#options' => ['' => $this->t('- None -')] + $styles,
      ];
    }

    return $container;
  }

  /**
   * Build responsive_image_style options for respective
   * field settings.
   *
   * @return array
   *  Array of options for select type.
   */
  public function getResponsiveStylesImage(): array {
    $site_styles = $this->entityTypeManager->getStorage('responsive_image_style')
      ->loadMultiple();
    $image_style = [];
    if (!empty($site_styles)) {
      $image_style[''] = t('- None -');
    }
    foreach ($site_styles as $key => $style) {
      $image_style[$key] = $key;
    }
    return $image_style;
  }

  /**
   * Return renderable array of image styles
   *
   * @param integer $count
   *
   * @return array
   */

  public function generateResponsiveStyleSelect(
    int $count,
    array $values = []
  ) {
    $styles = $this->getResponsiveStylesImage();
    $container = [];

    for ($i = 0; $i < $count; $i++) {
      $value = NULL;

      if (empty($values)) {
        $value = @$this->configuration['responsive_image_rendering']["responsive_style_$i"];
      }
      else {
        $value = $values['responsive_image_rendering']['styles']["responsive_style_$i"];
      }
      $container["responsive_style_$i"] = [
        '#type' => 'select',
        '#title' => $this->t('Responsive image style for post @count',
          ['@count' => $i]),
        '#default_value' => !empty($value) ? $value : NULL,
        '#options' => $styles,
      ];
    }

    return $container;
  }

  /**
   * Form ajax callback.
   *
   * @param array $form
   * @param FormStateInterface $form_state
   *
   * @return array
   */

  public function formSettingsAjax(
    array &$form,
    FormStateInterface $form_state
  ) {
    return $form['settings'];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['token'] = $form_state->getValue('token');
    $this->configuration['image_style_options'] = $form_state->getValue('media_settings')['image_style_options'];
    $this->configuration['image_style'] = $form_state->getValue('media_settings')['image_style'];
    $this->configuration['responsive_image_style'] = $form_state->getValue('media_settings')['responsive_image_style'];
    $this->configuration['limit'] = $form_state->getValue('media_settings')['limit'];
    $this->configuration['video'] = $form_state->getValue('media_settings')['video'];
    $this->configuration['autoplay'] = $form_state->getValue('media_settings')['autoplay'];
    $this->configuration['post_caption'] = $form_state->getValue('post_settings')['post_caption'];
    $this->configuration['insights'] = $form_state->getValue('post_settings')['insights'];
    $this->configuration['links'] = $form_state->getValue('post_settings')['links'];
    $this->configuration['header_mode'] = $form_state->getValue('post_settings')['header_mode'];
    $this->configuration['swiper'] = $form_state->getValue('styling_settings')['swiper'];
    $this->configuration['swiper_arrow'] = $form_state->getValue('styling_settings')['swiper_arrow'];
    $this->configuration['swiper_pagination'] = $form_state->getValue('styling_settings')['swiper_pagination'];
    $this->configuration['fancybox'] = $form_state->getValue('styling_settings')['fancybox'];
    $this->configuration['disabled_style'] = $form_state->getValue('styling_settings')['disabled_style'];
    $this->configuration['grid'] = $form_state->getValue('styling_settings')['grid'];
    $buildInfo = $form_state->getBuildInfo();

    $values = $form_state->getValues();
    $responsive_style_access = FALSE;

    if ($this->moduleHandler->moduleExists('responsive_image')) {
      $responsive_style_access = TRUE;
    }
    // Preset
    $this->configuration['different_image_style'] = $values['media_settings']['different_image_style'];
    if ($responsive_style_access) {
      $this->configuration['different_responsive_style'] = $values['media_settings']['different_responsive_style'];
    }

    // Image styles
    $this->configuration['image_styles_rendering'] = [];
    if ($this->configuration['different_image_style']) {
      $this->configuration['image_styles_rendering'] = $values['image_styles_rendering']['styles'];
    }

    // Responsive styles
    $this->configuration['responsive_image_rendering'] = [];
    if ($responsive_style_access && $this->configuration['different_responsive_style']) {
      $this->configuration['responsive_image_rendering'] = $values['responsive_image_rendering']['styles'];
    }

    $instagram_media_path = 'public://instagram_media/';

    if ($buildInfo['callback_object'] instanceof BlockForm) {
      $blockId = $buildInfo['callback_object']->getEntity()->id();
      $instagram_media_path .= $blockId . '/';
      $this->configuration['block_id'] = $blockId;
    }
    $this->configuration['media_path'] = $instagram_media_path;

    parent::blockSubmit($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    return AccessResult::allowedIfHasPermission($account, 'access content');
  }

}